﻿using System.Drawing;

namespace s4_visualizer
{
    class Renderer
    {
        public Renderer(System.Windows.Forms.PictureBox _surface)
        {
            surface = _surface;
            renderAreaSz = surface.Height;
            buffer = new Bitmap[2];
            buffer[0] = new Bitmap(renderAreaSz, renderAreaSz);
            buffer[1] = new Bitmap(renderAreaSz, renderAreaSz);
            backBuffer = 0;
            
            // define the colors for the different points
            colorArr = new Color[MAX_POINTS];

            for (int r = 0; r < 7; r++)
            {
                for (int g = 0; g < 7; g++)
                {
                    for (int b = 0; b < 7; b++)
                    {
                        colorArr[b * 49 + g * 7 + r] = Color.FromArgb(64 + r * 16, 64 + g * 16, 64 + b * 16);
                    }
                }
            }

            gridColor = Color.Black;
            emptyColor = Color.White;
            blockedColor = Color.Black;
        }

        public void draw (int[,] grid, int gridSz)
        {
            float mult = (float)gridSz / (float)renderAreaSz;

            for (int x = 0; x < renderAreaSz; x++)
            {
                for (int y = 0; y < renderAreaSz; y++)
                {
                    int cell = grid[(int)(x * mult), (int)(y * mult)];
                    if (cell > 0)
                    {
                        buffer[backBuffer].SetPixel(x, y, colorArr[cell]);
                    }
                    else if (cell == 0)
                    {
                        buffer[backBuffer].SetPixel(x, y, emptyColor);
                    }
                    else if (cell == -1)
                    {
                        buffer[backBuffer].SetPixel(x, y, blockedColor);
                    }
                }
            }

            drawGrid(gridSz);

            setImage(backBuffer);

            //swap the buffers for the next draw call
            backBuffer = backBuffer ^ 1;
        }

        public Point getClickedCell (int mouseX, int mouseY, int gridSz)
        {
            float mult = (float)gridSz / (float)renderAreaSz;
            return new Point((int)(mouseX * mult), (int)(mouseY * mult));
        }

        private void drawGrid (int gridSz)
        {
            float mult = (float)renderAreaSz / (float)gridSz;

            for (int i = 0; i < gridSz; i++)
            {
                float x = (float)i * mult;
                int linePix = (int)(x - 0.000001f);

                for (int j = 0; j < renderAreaSz; j++)
                {
                    buffer[backBuffer].SetPixel(j, linePix, Color.Black);
                    buffer[backBuffer].SetPixel(linePix, j, Color.Black);
                }
            }
        }

        public void setImage (int backBuffer)
        {
            surface.Invoke(new setImageDelegate(setImageSafe), backBuffer);
        }

        private void setImageSafe (int backBuffer)
        {
            surface.Image = buffer[backBuffer];
        }

        delegate void setImageDelegate(int backBuffer);

        private System.Windows.Forms.PictureBox surface;
        private int renderAreaSz;
        private Bitmap[] buffer;
        private int backBuffer;
        private const int MAX_POINTS = 1000;
        private Color[] colorArr;
        private Color emptyColor;
        private Color gridColor;
        private Color blockedColor;
    }
}